const clean = require('gulp-clean');
const cheerio = require('cheerio');
const fs = require('fs');
const gulp = require('gulp');

const aws = JSON.parse(fs.readFileSync('awsaccess.secret'));
const s3 = require('gulp-s3-upload')(aws);

const prepare = require('./prepare');
const utility = require('./utility');


// Output structure
//  / build
//      / meta
//      / data
//          / dev
//          / web
//      / html
//          / dev
//          / web
//      / state


// task:markdown
// task:annotate
require('./tasks/markdown');

// task:data-dev
// task:data-web
require('./tasks/data');



// task:index-codex
// task:codex
require('./tasks/indexing');

// task:template
// task:static
require('./tasks/template');

// Copy public files into the public folder
gulp.task('public', ['clean'], function() {
	return gulp.src('./public/**/*')
		.pipe(gulp.dest('../build/html/dev/public'))
		.pipe(gulp.dest('../build/html/web/public'));
});

gulp.task('visualizations', ['clean'], function() {
	return gulp.src('../visualizations/**/*')
		.pipe(gulp.dest('../build/html/dev/visualizations'))
		.pipe(gulp.dest('../build/html/web/visualizations'));
});

gulp.task('assets', ['public', 'visualizations']);

gulp.task('images', ['clean'], function() {
	return gulp.src('../writing/**/*.jpg')
		.pipe(gulp.dest('../build/html/dev'))
		.pipe(gulp.dest('../build/html/web'));
});

// Upload all build items 
gulp.task('publish', ['template', 'public', 'assets'], function() {
	return gulp.src(['../build/html/web/**/*', '!../build/html/web/meta/**/*'])
		.pipe(s3({
			Bucket: 'aaronsun.com',
			ACL: 'public-read',
			charset: 'utf-8',
			mimeTypeLookup: function(name) {
				if (name.indexOf('.') === -1) {
					return name + '.html';
				}
				return name;
			},
		}));
});

gulp.task('clean', function () {
	return gulp.src(['../build'], {read: false})
		.pipe(clean({force:true})); 
});

gulp.task('persist-clean', function () {
	return gulp.src('../html', {read: false})
		.pipe(clean({force:true}));
});

gulp.task('persist', ['persist-clean', 'template', 'assets', 'static'], function() {
	return gulp.src('../build/html/dev/**/*')
		.pipe(gulp.dest('../html'))
});

// Default Task
gulp.task('default', ['template', 'assets', 'static']);