// Preparation of a dom tree
const path = require('path');

const utility = require('./utility');

class Prepare {

	// demote links from markdown headers to page headers
	static markdown($) {
		// modify headers

		// change links
		$('h4').each((i, e) => {
			e.name = 'h6';
		});
		$('h3').each((i, e) => {
			e.name = 'h5';
		});
		$('h2').each((i, e) => {
			e.name = 'h4';
		});
		$('h1').each((i, e) => {
			e.name = 'h3';
		});
	}

	// process links and add in short cuts
	static links($) {
		$('a').each((i, e) => {
			const $e = $(e);
			const href = $e.attr('href');

			if (href == '') {

				// for links without a url, link to the page with the same name as the contents.
				$e.attr('href', utility.titlefy($e.text()));

			}
		});
	}

	static links2($, data, index) {
		$('a').each((i, e) => {
			const $e = $(e);
			let href = $e.attr('href');

			if (index[href]) {
				href = path.join(data.base, index[href]);
			}

			if (href.endsWith('/index')) {
				href = href.substr(0, href.length - 'index'.length) + data.index;
			} else if (!path.basename(href).includes('.')) {

				// for links without an extension, add the default extension if necessary.
				href = href + data.extname;

			}

			$e.attr('href', href);
		});
	}

	static rebaseLinks($, former, latter) {
		$('a').each((i, e) => {
			const $e = $(e);
			let href = $e.attr('href');
			if (href.startsWith(former)) {
				href = latter + href.substr(former.length);
			}
			$e.attr('href', href);
		});
	}

	static code($) {
		$('pre > code').each((i, e) => {
			const $e = $(e);

			if ($e.html().includes('\n')) {
				$e.parent().addClass('code');
			}
		});
	}

	// properly format blockquotes
	static blockquotes($) {
		
		$('blockquote ul > li:only-child').parent().each((i, e) => {
			const $e = $(e);
			$e.replaceWith($(`<cite>—${$e.text()}</cite>`));
		});

		$('blockquote').each((i, e) => {
			const $b = $(e);

			$b.children(':first-child, cite + p').each((i, e) => {
				const $e = $(e);
				const g = $e.nextUntil('cite + p').add($e);
				const c = $('<blockquote></blockquote>');


				c.append(g.remove()).insertBefore($b);

			});

			$b.remove();
		});

		$('blockquote ul').remove();
	}

	// make images responsive sized
	static images($, data) {

		$('img').each((i, e) => {
			const $e = $(e);
			const src = $e.attr('src');
			const link = src.substr(src.indexOf(':') + 1)
			

			if (src.startsWith('youtube:')) {
				const youtube = $(`<div class="video-responsive"><iframe src="https://www.youtube.com/embed/${link}?rel=0" frameborder="0" allowfullscreen></iframe></div>`);
				$e.replaceWith(youtube);
			} else if (src.startsWith('embed:')) {
				const embed = $(`<div class="video-responsive video-responsive-1-1"><iframe src="${path.join(data.base, link)}" frameborder="0"></iframe></div>`);
				$e.replaceWith(embed);
			} else {
				$e.addClass('img-responsive');
			}
		});

		// Swap out image tags that are youtube links with the embeds
		// https://www.youtube.com/embed/XL3Xc5vd_Bg?rel=0

		// <iframe width="560" height="315" src="https://www.youtube.com/embed/XL3Xc5vd_Bg?rel=0" frameborder="0" allowfullscreen></iframe>

	}


	static indexHeaders($) {
    	// iterate through headers
	    const headers = $('h1, h2, h3, h4, h5');
	    const tree = { children: [] };
	    const levels = [tree];

	    headers.each((i, e) => {
	        const level = e.name.substr(1) * 1;
	        const $e = $(e);
	        const text = $e.text();
	        const id = utility.idify(text);

	        $e.attr('id', id);

	        const node = {
	            text,
	            id,
	            children: [],
	        }
	        if (levels[level - 1])
	            levels[level - 1].children.push(node);
	        levels[level] = node;
	    });

	    const rootNode = tree.children[0];
	    if (rootNode) {
	        if (tree.children.length == 1 &&
	            rootNode.children.length == 0) {
	            // don't build an outline if there's only one element
	            return [];
	        }

	        rootNode.text = "Outline";
	    }

	    return tree.children;
	}


	// Processing on index documents (left handed navigation)
	static index($, data) {
		$('ul').addClass('nav');
		$('li').addClass('nav-item'); 
	}
}

module.exports = Prepare;