

const gulp = require('gulp');
const cheerio = require('cheerio');
const metaMarkdown = require('gulp-meta-markdown');
const rename = require('gulp-rename');
const highlight = require('highlight.js');

const utility = require('../utility');
const prepare = require('../prepare');


// generate html content from markdown
gulp.task('markdown', ['clean'], () => {
    // API can be found at https://github.com/chjj/marked 
    metaMarkdown.marked.setOptions({
		highlight: function (code) {
			return highlight.highlightAuto(code).value;
		}
    });

	return gulp.src('../writing/**/*.md')
		.pipe(metaMarkdown())
		.pipe(rename(p => {
			p.basename = utility.titlefy(p.basename);
		}))
		.pipe(gulp.dest('../build/meta'));
});

// generate heading indices from html
gulp.task('annotate', ['markdown'], () => {
	return gulp.src('../build/meta/**/*.json')
		.pipe(utility.modifyJson((data) => {
			const $ = cheerio.load(data.content);

			data.headers = prepare.indexHeaders($);

			const title = $('h1').first();
			data.title = title.text();
			data.id = utility.idify(data.title);
			title.remove();

			prepare.markdown($);
			prepare.links($);
			prepare.blockquotes($);
			prepare.code($);

			data.content = $('body').html();

			// additions scripts
			if (data.content.includes("\\(") || data.content.includes('\\[')) {
				data.scripts = [
					'https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.2/MathJax.js?config=TeX-AMS_CHTML'
				];
			}
		}))
		.pipe(gulp.dest('../build/meta'));
});