
const gulp = require('gulp');
const path = require('path');
const through = require('through2');
const handlebars = require('gulp-hb');
const cheerio = require('cheerio');
const rename = require('gulp-rename');

const prepare = require('../prepare');
const utility = require('../utility');

const handlebarHelpers = {
	'path-join': function() {
		return path.join.apply(null, Array.prototype.slice.call(arguments, 0, arguments.length - 1));
	},
	'eq': (a, b) => a == b,
	'log' : (a) => { console.log(a); return a }
};

gulp.task('template', ['index', 'data'], function() {
	return gulp
		.src('../build/data/**/*.json')
		.pipe(through.obj(function(file, enc, cb) {
			const data = JSON.parse(file.contents.toString());
			const relative = path.relative(file.base, file.path);
			const paths = relative.split(path.sep);

			// TODO: Bug where the relative links are wrong for second level nesting for dev
			const index = path.join(file.base, path.join.apply(null, paths.slice(0, 2)), "_.json");

			// get the relevant template file and data
			data.contents = utility.readJson(index);

			if (data.contents && data.base != data.contents.base) {
				const $ = cheerio.load(data.contents.content);
				prepare.rebaseLinks($, data.contents.base, data.base);
				data.contents.content = $('body').html();
			}

			gulp
				.src('./templates/*.html')
				.pipe(handlebars()
					.helpers(handlebarHelpers)
					.data(data))
				.pipe(rename(p => {
					p.dirname = path.dirname(relative);
					p.basename = path.basename(relative, '.json').toLowerCase();
					p.extname = p.basename == 'index' ? '.html' : data.extname; // no extension when uploading
				}))
				.pipe(gulp.dest('../build/html/'))
				.on('error', cb)
				.on('end', cb);
		}));
});

gulp.task('static', ['data', 'images'], function() {
	return gulp
		.src('../build/data/*/base.json')
		.pipe(through.obj(function(file, enc, cb) {
			const relative = path.relative(file.base, file.path);

			const data = JSON.parse(file.contents.toString());
			data.dist = relative.substring(0, relative.indexOf(path.sep));

			gulp
				.src('./templates/pages/**/*.html')
				.pipe(handlebars()
					.helpers(handlebarHelpers)
					.data(data))
				.pipe(rename(p => {
					const relative = path.relative(file.base, file.path);
					p.dirname = path.dirname(relative);
					p.basename = p.basename.toLowerCase();
					p.extname = p.basename == 'index' ? '.html' : data.extname; // no extension when uploading
				}))
				.pipe(gulp.dest('../build/html/'))
				.on('error', cb)
				.on('end', cb);
		}));
});