const through = require('through2');
const fs = require('fs');



// Converts text to a html id name
const idify = id => id.replace(/\W+/g, '-').toLowerCase();

// Converts text to a file name 
const titlefy = title => idify(title).replace(/\W+$/,'');

const modifyJson = (mod) => {
	return through.obj(function(file, enc, callback) {
		// parse file contents
		const data = JSON.parse(file.contents.toString());

		// modify the JSON object
		const revised = mod(data, file);

		// create a new file with the revised object
		const output = file.clone();
		output.contents = new Buffer(
			JSON.stringify(revised !== undefined ? revised : data, null, '\t'));
		this.push(output);
		callback();
	});
};

/**
 * Tries to read a json file if it exists, otherwise returns the default.
 */
const readJson = function(path, defaultValue) {
	if (fs.existsSync(path)) {
		return JSON.parse(fs.readFileSync(path));
	}
	return defaultValue;
};

const sortObject = (obj) => {
	const keys = Object.keys(obj).sort();
	const sorted = {};

	for (key of keys) {
		sorted[key] = obj[key];
	}

	return sorted;
}

module.exports = { idify, titlefy, modifyJson, readJson, sortObject };